@@ -3,10 +3,10 @@
 
   rbgtkwidget.c -
 
-  $Author: sakai $
-  $Date: 2003/11/20 18:27:54 $
+  $Author: mutoh $
+  $Date: 2004/03/05 16:24:30 $
 
-  Copyright (C) 2002,2003 Ruby-GNOME2 Project Team
+  Copyright (C) 2002-2004 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
                           Daisuke Kanda,
                           Hiroshi Igarashi
@@ -16,6 +16,8 @@
 
 #define _SELF(self) (GTK_WIDGET(RVAL2GOBJ(self)))
 
+static VALUE style_prop_func_table;
+
 static VALUE
 widget_get_flags(self)
     VALUE self;
@@ -682,41 +684,111 @@ widget_mnemonic_activate(self, group_cycling)
     return gtk_widget_mnemonic_activate(_SELF(self), RTEST(group_cycling)) ? Qtrue : Qfalse;
 }
 
-/* Can we implement this?
-static VALUE
-widget_class_install_style_property(self, pspec)
-    VALUE self, pspec;
+/*
+ * Gtk::Widget.install_style_property(...) do |spec, str|
+ *   #parse str
+ *   new_val = ....   # parse value from string.
+ *   new_val          # return new_val or nil if you can't convert the value.
+ * end
+ */
+static gboolean
+rc_property_parser(pspec, rc_string, property_value)
+    const GParamSpec *pspec;
+    const GString *rc_string;
+    GValue *property_value;
 {
-    gtk_widget_class_install_style_property((GtkWidgetClass*)G_OBJECT_GET_CLASS(_SELF(self)), 
-                                            RVAL2GOBJ(pspec));
+    VALUE spec = GOBJ2RVAL((gpointer)pspec);
+    VALUE func = rb_hash_aref(style_prop_func_table, spec);
+    VALUE ret = rb_funcall(func, id_call, 2, spec, CSTR2RVAL(rc_string->str));
+    if (NIL_P(ret)) {
+        return FALSE;
+    } else if (RTEST(ret)){
+        rbgobj_rvalue_to_gvalue(ret, property_value);
+        return TRUE;
+    } else {
+        rb_raise(rb_eArgError, "Gtk::Widget#install_style_property() block should return new value or nil");
+    }
+}
+
+static VALUE
+widget_s_install_style_property(self, spec)
+    VALUE self, spec;
+{
+    const RGObjClassInfo* cinfo = rbgobj_lookup_class(self);
+    GtkWidgetClass* gclass;
+    GParamSpec* pspec = G_PARAM_SPEC(RVAL2GOBJ(spec));
+
+    if (cinfo->klass != self)
+        rb_raise(rb_eTypeError, "%s isn't registered class",
+                 rb_class2name(self));
+
+    gclass = (GtkWidgetClass *)g_type_class_ref(cinfo->gtype);
+    if (rb_block_given_p()){
+        VALUE func = G_BLOCK_PROC();
+        rb_hash_aset(style_prop_func_table, spec, func);
+        gtk_widget_class_install_style_property_parser(gclass, pspec, rc_property_parser);
+    } else {
+        gtk_widget_class_install_style_property(gclass, pspec); 
+    }
     return self;
 }
 
-void        gtk_widget_class_install_style_property_parser
-                                            (GtkWidgetClass *klass,
-                                             GParamSpec *pspec,
-                                             GtkRcPropertyParser parser);
-GParamSpec* gtk_widget_class_find_style_property
-                                            (GtkWidgetClass *klass,
-                                             const gchar *property_name);
-*/
 #if GTK_CHECK_VERSION(2,2,0)
 static VALUE
-widget_style_properties(self)
+widget_s_find_style_property(self, property_name)
+     VALUE self, property_name;
+{
+    GtkWidgetClass* oclass;
+    const char* name;
+    GParamSpec* prop;
+    VALUE result;
+
+    if (SYMBOL_P(property_name)) {
+        name = rb_id2name(SYM2ID(property_name));
+    } else {
+        StringValue(property_name);
+        name = StringValuePtr(property_name);
+    }
+
+    oclass = (GtkWidgetClass*)g_type_class_ref(CLASS2GTYPE(self));
+
+    prop = gtk_widget_class_find_style_property(oclass, name);
+    if (!prop){
+        g_type_class_unref(oclass);
+        rb_raise(rb_eval_string("GLib::NoPropertyError"), "no such property: %s", name);
+    }
+
+    result = GOBJ2RVAL(prop);
+    g_type_class_unref(oclass);
+    return result;
+}
+
+static VALUE
+widget_s_style_properties(argc, argv, self)
+    int argc;
+    VALUE* argv;
     VALUE self;
 {
     GtkWidgetClass* oclass = g_type_class_ref(CLASS2GTYPE(self));
     gint n_properties;
     GParamSpec** props;
+    VALUE inherited_too;
     VALUE ary;
     int i;
 
+    if (rb_scan_args(argc, argv, "01", &inherited_too) == 0)
+        inherited_too = Qtrue;
+
     props = gtk_widget_class_list_style_properties(oclass, &n_properties);
 
-    ary = rb_ary_new2(n_properties);
-    for (i = 0; i < n_properties; i++)
-        rb_ary_store(ary, i, rb_str_new2(props[i]->name));
+    ary = rb_ary_new();
+    for (i = 0; i < n_properties; i++){
+        if (RTEST(inherited_too)
+            || GTYPE2CLASS(props[i]->owner_type) == self)
+            rb_ary_push(ary, rb_str_new2(props[i]->name));
+    }
 
+    g_free(props);
     g_type_class_unref(oclass);
     return ary;
 }
@@ -751,9 +823,8 @@ static VALUE
 widget_style_get_property(self, prop_name)
     VALUE self, prop_name;
 {
-    GParamSpec* pspec;
+    GParamSpec* pspec = NULL;
     const char* name;
-    GtkWidgetClass* oclass = g_type_class_ref(RVAL2GTYPE(self));
 
     if (SYMBOL_P(prop_name)) {
         name = rb_id2name(SYM2ID(prop_name));
@@ -762,18 +833,17 @@ widget_style_get_property(self, prop_name)
         name = StringValuePtr(prop_name);
     }
 #if GTK_CHECK_VERSION(2,2,0)
-    pspec = gtk_widget_class_find_style_property(oclass, name);
+    pspec = gtk_widget_class_find_style_property((GtkWidgetClass*)g_type_class_ref(RVAL2GTYPE(self)), name);
 #endif
     if (!pspec)
         rb_raise(rb_eval_string("GLib::NoPropertyError"), "No such property: %s", name);
     else {
         // FIXME: use rb_ensure to call g_value_unset()
-        GValueToRValueFunc getter = NULL;
         GValue gval = {0,};
         VALUE ret;
         g_value_init(&gval, G_PARAM_SPEC_VALUE_TYPE(pspec));
         gtk_widget_style_get_property(GTK_WIDGET(RVAL2GOBJ(self)), name, &gval);
-        ret = getter ? getter(&gval) : GVAL2RVAL(&gval);
+        ret = GVAL2RVAL(&gval);
         g_value_unset(&gval);
         return ret;
     }
@@ -949,6 +1019,9 @@ Init_gtk_widget()
 {
     VALUE gWidget = G_DEF_CLASS(GTK_TYPE_WIDGET, "Widget", mGtk);
 
+    rb_global_variable(&style_prop_func_table);
+    style_prop_func_table = rb_hash_new();
+
     /*
      * instance methods
      */
@@ -977,8 +1050,10 @@ Init_gtk_widget()
     rb_define_method(gWidget, "event", widget_event, 1);
     rb_define_method(gWidget, "activate", widget_activate, 0);
     rb_define_method(gWidget, "reparent", widget_reparent, 1);
+    rb_define_singleton_method(gWidget, "install_style_property", widget_s_install_style_property, 1);
 #if GTK_CHECK_VERSION(2,2,0)
-    rb_define_singleton_method(gWidget, "style_properties", widget_style_properties, 0);
+    rb_define_singleton_method(gWidget, "style_property", widget_s_find_style_property, 1);
+    rb_define_singleton_method(gWidget, "style_properties", widget_s_style_properties, -1);
 #endif
     rb_define_method(gWidget, "intersect", widget_intersect, 1);
     rb_define_method(gWidget, "focus?", widget_is_focus, 0);
